@@ -2,25 +2,24 @@ import typing as t
 import weakref
 from contextlib import contextmanager
 from enum import Flag, auto
-from functools import partial
 from pathlib import Path
 from warnings import warn
 
 from pyqtgraph.Qt import QtCore, QtWidgets
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
-from utilitys.fns import applyParamOpts
 
 from . import PrjParam
 from .procwrapper import NestedProcWrapper
-from ..fns import nameFormatter, clsNameOrGroup, saveToFile, clearUnwantedParamVals, paramValues, \
-  params_flattened, flexibleParamTree, resolveYamlDict, getParamChild, dialogGetSaveFileName, docParser, paramDictWithOpts
+from .. import fns
 from ..processing import *
 from ..typeoverloads import FilePath
-from ..widgets import EasyWidget
+import utilitys
+from ..fns import params_flattened, warnLater
 
 __all__ = [
-  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS'
+  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS',
+  'params_flattened'
 ]
 
 Signal = QtCore.Signal
@@ -42,7 +41,7 @@ def _mkRunDict(proc: ProcessStage, btnOpts: t.Union[PrjParam, dict]):
     btnOpts.pop('type', None)
     defaultBtnOpts.update(btnOpts)
   if len(defaultBtnOpts.get('tip', '')) == 0 and isinstance(proc, AtomicProcess):
-    defaultBtnOpts['tip'] = docParser(proc.func.__doc__)['top-descr']
+    defaultBtnOpts['tip'] = fns.docParser(proc.func.__doc__)['top-descr']
   if len(proc.input.hyperParamKeys) > 0:
     # In this case, a descriptive name isn't needed since the func name will be
     # present in the parameter group
@@ -95,7 +94,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     if paramList is None:
       paramList = []
     if name is None:
-      name = nameFormatter(clsNameOrGroup(cls).replace('Editor', ''))
+      name = fns.nameFormatter(fns.clsNameOrGroup(cls).replace('Editor', ''))
 
     if saveDir is not None:
       saveDir = Path(saveDir)
@@ -128,14 +127,13 @@ class ParamEditor(QtWidgets.QDockWidget):
     # Construct parameter tree
     # -----------
     self.params = Parameter.create(name='Parameters', type='group', children=paramList)
-    self.tree = flexibleParamTree()
 
     self.params.sigStateChanged.connect(self._paramTreeChanged)
 
     topParam = self.params
     if topTreeChild is not None:
       topParam = topTreeChild
-    self.tree.setParameters(topParam, showTop=False)
+    self.tree = fns.flexibleParamTree(topParam, showTop=False)
 
     self._stateBeforeEdit = self.params.saveState()
     self.lastAppliedName = None
@@ -158,7 +156,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     return outDict
 
   def saveParamValues(self, saveName: str=None, paramState: dict=None, allowOverwriteDefault=False,
-                      includeDefaults=False, blockWrite=False, includeKeys=()):
+                      includeDefaults=False, blockWrite=False):
     """
     * Returns dict on successful parameter save and emits sigParamStateCreated.
     * Returns None if no save name was given
@@ -166,37 +164,11 @@ class ParamEditor(QtWidgets.QDockWidget):
     if saveName is None or self.saveDir is None:
       return None
     elif paramState is None:
-      if includeKeys:
-        paramState = paramDictWithOpts(self.parmas, includeKeys)
-      else:
-        paramState = paramValues(self.params, includeDefaults=includeDefaults or allowOverwriteDefault)
+      paramState = fns.paramValues(self.params, includeDefaults=includeDefaults or allowOverwriteDefault)
     # Remove non-useful values
     if not blockWrite and self.saveDir is not None:
       self.saveDir.mkdir(parents=True, exist_ok=True)
-      saveToFile(paramState, self.formatFileName(saveName),
-                 allowOverwriteDefault=allowOverwriteDefault)
-    # self.applyChanges()
-    self.lastAppliedName = saveName
-    self.sigParamStateCreated.emit(str(saveName))
-    return paramState
-
-  def saveParamState(self, saveName: str=None, paramState: dict=None,
-                     allowOverwriteDefault=False, includeDefaults=False, blockWrite=False):
-    """
-    * Returns dict on successful parameter save and emits sigParamStateCreated.
-    * Returns None if no save name was given
-    """
-    if saveName is None or self.saveDir is None:
-      return None
-    elif paramState is None:# and allowOverwriteDefault:
-      paramState = self.params.saveState(filter='user')
-      # clearUnwantedParamVals(paramState)
-    # else:
-    #   paramState = paramValues(self.params, includeDefaults=includeDefaults)
-    # Remove non-useful values
-    if not blockWrite and self.saveDir is not None:
-      self.saveDir.mkdir(parents=True, exist_ok=True)
-      saveToFile(paramState, self.formatFileName(saveName),
+      fns.saveToFile(paramState, self.formatFileName(saveName),
                  allowOverwriteDefault=allowOverwriteDefault)
     # self.applyChanges()
     self.lastAppliedName = saveName
@@ -204,15 +176,14 @@ class ParamEditor(QtWidgets.QDockWidget):
     return paramState
 
   def saveCurStateAsDefault(self):
-    self.saveParamValues('Default', allowOverwriteDefault=True)
-    self._setParamTooltips()
+    return self.saveParamValues('Default', allowOverwriteDefault=True)
 
   def _loadParamState(self, stateName: str, stateDict: dict=None, applyChanges=True):
     # Bug in pyqtgraph restore state doesn't play nice when parameters have connected functions outside the parameter
     # item, so re-implement without inserting or removing children
     loadDict = self._parseStateDict(stateName, stateDict)
     with self.params.treeChangeBlocker():
-      applyParamOpts(self.params, loadDict)
+      fns.applyParamOpts(self.params, loadDict)
     if applyChanges:
       self.applyChanges()
     self.lastAppliedName = stateName
@@ -233,7 +204,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       return self._loadParamState(stateName, stateDict)
     
     if candidateParams is None:
-      candidateParams = params_flattened(self.params)
+      candidateParams = fns.params_flattened(self.params)
       
     def validName(param, name):
       return name in (param.opts['title'], param.name())
@@ -257,28 +228,33 @@ class ParamEditor(QtWidgets.QDockWidget):
         if len(matches) == 1:
           matches[0].setValue(vv)
         elif len(matches) == 0:
-          warn(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
+          warnLater(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
         else:
           raise ValueError(f'Multiple matching parameters for key {kk}:\n'
                            f'{matches}')
     if applyChanges:
       self.applyChanges()
     self.lastAppliedName = stateName
-    return paramValues(self.params, includeDefaults=True)
+    return fns.paramValues(self.params, includeDefaults=True)
 
 
   def formatFileName(self, stateName: t.Union[str, Path]=None):
     if stateName is None:
       stateName = self.lastAppliedName
+    suffix = f'.{self.fileType}'
+    stateName = stateName or suffix
     stateName = Path(stateName)
-    if stateName.is_absolute():
-      staPathPlusStem = stateName
-    else:
-      staPathPlusStem = self.saveDir/stateName
-    return staPathPlusStem.with_suffix(f'.{self.fileType}')
+    if self.saveDir is None:
+      # Prevents ValueError at the final return
+      return stateName
+    elif not stateName.is_absolute():
+      stateName = self.saveDir / stateName
+    if not stateName.suffix:
+      stateName = stateName.with_suffix(suffix)
+    return stateName
 
   def _parseStateDict(self, stateName: t.Union[str, Path], stateDict: dict=None):
-    return resolveYamlDict(self.formatFileName(stateName), stateDict)[1]
+    return fns.resolveYamlDict(self.formatFileName(stateName), stateDict)[1]
 
   def deleteParamState(self, stateName: str):
     filename = self.formatFileName(stateName)
@@ -327,7 +303,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       namePath = tuple(self._baseRegisterPath) + tuple(namePath)
     else:
       namePath = tuple(overrideBasePath) + tuple(namePath)
-    paramForEditor = getParamChild(self.params, *namePath, chOpts=paramOpts)
+    paramForEditor = fns.getParamChild(self.params, *namePath, chOpts=paramOpts)
 
     self.registeredPrjParams.append(constParam)
     if not asProperty:
@@ -399,10 +375,10 @@ class ParamEditor(QtWidgets.QDockWidget):
     else:
       namePath = tuple(overrideBasePath) + tuple(namePath)
 
-    topParam = getParamChild(self.params, *namePath)
+    topParam = fns.getParamChild(self.params, *namePath)
     if len(proc.input.hyperParamKeys) > 0:
       # Check if proc params already exist from a previous addition
-      wrapped = NestedProcWrapper(proc, topParam, paramFormat or nameFormatter, treatAsAtomic=True, nestHyperparams=nest)
+      wrapped = NestedProcWrapper(proc, topParam, paramFormat or fns.nameFormatter, treatAsAtomic=True, nestHyperparams=nest)
       parentParam = wrapped.parentParam
       for param in parentParam:
         if runOpts & RunOpts.ON_CHANGED:
@@ -416,14 +392,10 @@ class ParamEditor(QtWidgets.QDockWidget):
       if not nest:
         # Make sure button name is correct
         runBtnDict['name'] = proc.name
-      runBtn = getParamChild(parentParam, chOpts=runBtnDict)
+      runBtn = fns.getParamChild(parentParam, chOpts=runBtnDict)
       runBtn.sigActivated.connect(runProc)
     if runOpts & RunOpts.ON_APPLY:
       self.sigChangesApplied.connect(runProc)
-    try:
-      self.setParamTooltips(False)
-    except AttributeError:
-      pass
     self.procToParamsMapping[proc] = parentParam
 
     if returnParam:
@@ -432,14 +404,14 @@ class ParamEditor(QtWidgets.QDockWidget):
 
   @classmethod
   def buildClsToolsEditor(cls, forCls: type, name: str=None):
-    groupName = clsNameOrGroup(forCls)
+    groupName = fns.clsNameOrGroup(forCls)
     lowerGroupName = groupName.lower()
     if name is None:
       name = groupName
-    if not groupName.endswith('Tools'):
-      name = groupName + ' Tools'
+    if not name.endswith('Tools'):
+      name = name + ' Tools'
     toolsEditor = cls(saveDir=None,
-                              fileType=lowerGroupName.replace(' ', '') + 'toolsEditor',
+                              fileType=lowerGroupName.replace(' ', ''),
                               name=name, useNewInit=False)
     for btn in (toolsEditor.saveAsBtn, toolsEditor.applyBtn, toolsEditor.expandAllBtn,
                 toolsEditor.collapseAllBtn):
@@ -540,7 +512,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     ]
     if self.saveDir is None:
       self.saveAsBtn.hide()
-    self.dockContentsWidget = EasyWidget.buildWidget(children)
+    self.dockContentsWidget = utilitys.widgets.EasyWidget.buildWidget(children)
     self.centralLayout = self.dockContentsWidget.easyChild.layout_
     self.setWidget(self.dockContentsWidget)
 
@@ -550,36 +522,11 @@ class ParamEditor(QtWidgets.QDockWidget):
     # -----------
     # UI Element Signals
     # -----------
-    self.expandAllBtn.clicked.connect(lambda: self._setParamsExpanded())
-    self.collapseAllBtn.clicked.connect(lambda: self._setParamsExpanded(False))
+    self.expandAllBtn.clicked.connect(lambda: fns.setParamsExpanded(self.tree))
+    self.collapseAllBtn.clicked.connect(lambda: fns.setParamsExpanded(self.tree, False))
     self.saveAsBtn.clicked.connect(self.saveParamValues_gui)
     self.applyBtn.clicked.connect(self.applyChanges)
 
-  def _setParamTooltips(self, expandNameCol=True):
-    iterator = QtWidgets.QTreeWidgetItemIterator(self.tree)
-    item: QtWidgets.QTreeWidgetItem = iterator.value()
-    while item is not None:
-      # TODO: Set word wrap on long labels. Currently either can show '...' or wrap but not
-      #   both
-      # if self.tree.itemWidget(item, 0) is None:
-      #   lbl = QtWidgets.QLabel(item.text(0))
-      #   self.tree.setItemWidget(item, 0, lbl)
-      if (hasattr(item, 'param')
-          and 'tip' in item.param.opts
-          and len(item.toolTip(0)) == 0
-          and self.tree.itemWidget(item, 0) is None):
-        item.setToolTip(0, item.param.opts['tip'])
-      iterator += 1
-      item = iterator.value()
-    if expandNameCol:
-      self._setParamsExpanded(True)
-
-  def _setParamsExpanded(self, expandedVal=True):
-    for item in self.tree.topLevelItems():
-      for ii in range(item.childCount()):
-        item.child(ii).setExpanded(expandedVal)
-    self.tree.resizeColumnToContents(0)
-
   def __repr__(self):
     selfCls = type(self)
     oldName: str = super().__repr__()
@@ -611,12 +558,8 @@ class ParamEditor(QtWidgets.QDockWidget):
     self.params.restoreState(self._stateBeforeEdit, removeChildren=False)
     super().reject()
 
-  def saveParamState_gui(self):
-    saveName = dialogGetSaveFileName(self, 'Save As', self.lastAppliedName)
-    self.saveParamState(saveName)
-
   def saveParamValues_gui(self):
-    saveName = dialogGetSaveFileName(self, 'Save As', self.lastAppliedName)
+    saveName = fns.dialogGetSaveFileName(self, 'Save As', self.lastAppliedName)
     self.saveParamValues(saveName)
 
 class ParamEditorDockGrouping(QtWidgets.QDockWidget):
@@ -704,7 +647,7 @@ class EditorPropsMixin:
   REGISTERED_GROUPINGS = set()
   def __new__(cls, *args, **kwargs):
     if cls.__groupingName__ is None:
-      cls.__groupingName__ = nameFormatter(cls.__name__)
+      cls.__groupingName__ = fns.nameFormatter(cls.__name__)
     if cls not in cls.REGISTERED_GROUPINGS:
       basePath = (cls.__groupingName__,)
       if basePath[0] == '':
