@@ -16,7 +16,7 @@ from pyqtgraph.parametertree.parameterTypes import ActionParameterItem, ActionPa
 from ..fns import clsNameOrGroup, params_flattened, popupFilePicker
 from .prjparam import PrjParam
 from .shortcut import ShortcutsEditor
-from ..widgets import PopupLineEditor
+from ..widgets import PopupLineEditor, ButtonCollection
 
 
 class MonkeyPatchedTextParameterItem(TextParameterItem):
@@ -110,14 +110,13 @@ class ShortcutParameterItem(ActionParameterItem):
     super().__init__(param, depth)
     btn: QtWidgets.QPushButton = self.button
     btn.setToolTip(param.opts.get('tip', ''))
+    self.setText(0, '')
     owner = param.opts.get('ownerObj', _Global)
     if param.value() is None or not ShortcutParameter.REGISTRY:
       return
     # Else: shortcut exists to be registered
-    _, param = ShortcutParameter.REGISTRY.createRegisteredButton(
-      PrjParam(**param.opts), baseBtn=self.button, namePath=(clsNameOrGroup(owner),), returnParam=True
-    )
-    self._postInit_(param)
+    prjOpts = PrjParam(**param.opts)
+    ButtonCollection.createBtn(prjOpts, baseBtn=self.button, namePath=(clsNameOrGroup(owner),))
 
   def _postInit_(self, shcParam: Parameter):
     """
