@@ -41,6 +41,9 @@ class ParamEditorPlugin(pe.EditorPropsMixin):
   _makeMenuShortcuts=True
   """Whether shortcuts should be added to menu options"""
 
+  _toolsEditorName:str=None
+  """Name of the class tools editor. If not provided, defaults to '<class name> Tools'"""
+
   win = None
   """Reference to the application main window"""
 
@@ -54,8 +57,13 @@ class ParamEditorPlugin(pe.EditorPropsMixin):
 
   @classmethod
   def __initEditorParams__(cls):
+    """
+    Creates most of the necessary components for interacting with editor properties including
+    a dock that can be placed in a main window, a tools editor for registering properties and
+    functions, and a dropdown menu for accessing these functions.
+    """
     cls.dock = pe.ParamEditorDockGrouping(dockName=cls.name)
-    cls.toolsEditor = pe.ParamEditor.buildClsToolsEditor(cls)
+    cls.toolsEditor = pe.ParamEditor.buildClsToolsEditor(cls, cls._toolsEditorName)
     if cls._showFuncDetails:
       cls.dock.addEditors([cls.toolsEditor])
     cls.menu = QtWidgets.QMenu(cls.name)
