@@ -196,7 +196,7 @@ class AtomicProcess(ProcessStage):
     self.func = func
 
   @classmethod
-  def _wrappedFunc(cls, func, mainResultKeys: StrList=None, mainInputKeys: StrList=None):
+  def _wrappedFunc(cls, func, mainResultKeys: StrList=None):
     """
     Wraps a function returining either a result or list of results, instead making the
     return value an `FRProcessIO` object where each `cls.mainResultkey` corresponds
