@@ -1,17 +1,25 @@
 from __future__ import annotations
 
+import typing as t
 from textwrap import wrap
-from typing import Union, Sequence, Any
-from typing_extensions import Literal
 
 import numpy as np
 import pandas as pd
 import pyqtgraph as pg
+import cv2 as cv
 from pyqtgraph.Qt import QtWidgets, QtGui, QtCore
 # Taken directly from https://stackoverflow.com/a/20610786/9463643
 from pyqtgraph.graphicsItems.ViewBox.ViewBoxMenu import ViewBoxMenu
+from pyqtgraph.parametertree import Parameter
+from typing_extensions import Literal
 
+from utilitys.misc import CompositionMixin
+from utilitys.typeoverloads import FilePath
+from . import params
 from .constants import PrjEnums
+from .params.prjparam import PrjParam
+from .params import parameditor
+
 try:
   from pyqtgraph.Qt import QtWidgets
   from qtconsole.rich_jupyter_widget import RichJupyterWidget
@@ -217,9 +225,150 @@ class PopupLineEditor(QtWidgets.QLineEdit):
     if self.text() == '':
       self.completer().setCompletionPrefix('')
 
-_layoutTypes = Union[Literal['H'], Literal['V']]
+
+class _DEFAULT_OWNER: pass
+"""None is a valid owner, so create a sentinel that's not valid"""
+btnCallable = t.Callable[[PrjParam], t.Any]
+class ButtonCollection(QtWidgets.QGroupBox):
+  def __init__(self, parent=None, title: str=None, btnParams: t.Collection[PrjParam]=(),
+               btnTriggerFns: t.Union[btnCallable, t.Collection[btnCallable]]=(),
+               exclusive=True, asToolBtn=True,
+               **createOpts):
+    super().__init__(parent)
+    self.lastTriggered: t.Optional[PrjParam] = None
+    self.uiLayout = QtWidgets.QHBoxLayout(self)
+    self.btnGroup = QtWidgets.QButtonGroup(self)
+    self.paramToFuncMapping: t.Dict[PrjParam, btnCallable] = dict()
+    self.paramToBtnMapping: t.Dict[PrjParam, QtWidgets.QPushButton] = dict()
+    self.asToolBtn = asToolBtn
+    if title is not None:
+      self.setTitle(title)
+    self.btnGroup.setExclusive(exclusive)
+
+    if not isinstance(btnTriggerFns, t.Iterable):
+      btnTriggerFns = [btnTriggerFns]*len(btnParams)
+    for param, fn in zip(btnParams, btnTriggerFns):
+      self.create_addBtn(param, fn, **createOpts)
+
+  def create_addBtn(self, btnParam: PrjParam, triggerFn: btnCallable, checkable=False, **createOpts):
+    if btnParam in self.paramToBtnMapping:
+      # Either already exists or wasn't designed to be a button
+      return
+    createOpts.setdefault('asToolBtn', self.asToolBtn)
+    newBtn = self.createBtn(btnParam, **createOpts)
+    if checkable:
+      newBtn.setCheckable(True)
+      oldTriggerFn = triggerFn
+      # If the button is checkable, only call this function when the button is checked
+      def newTriggerFn(param: PrjParam):
+        if newBtn.isChecked():
+          oldTriggerFn(param)
+      triggerFn = newTriggerFn
+    newBtn.clicked.connect(lambda: self.callFuncByParam(btnParam))
+
+    self.addBtn(btnParam, newBtn, triggerFn)
+    return newBtn
+
+  def clear(self):
+    for button in self.paramToBtnMapping.values():
+      self.btnGroup.removeButton(button)
+      self.uiLayout.removeWidget(button)
+      button.deleteLater()
+
+    self.paramToBtnMapping.clear()
+    self.paramToFuncMapping.clear()
+
+  def addFromExisting(self, other: ButtonCollection, which: t.Collection[PrjParam]=None):
+    for (param, btn), func in zip(other.paramToBtnMapping.items(), other.paramToFuncMapping.values()):
+      if which is None or param in which:
+        self.addBtn(param, btn, func)
+
+  def addBtn(self, param: PrjParam, btn: QtWidgets.QPushButton, func: btnCallable):
+    self.btnGroup.addButton(btn)
+    self.uiLayout.addWidget(btn)
+    self.paramToFuncMapping[param] = func
+    self.paramToBtnMapping[param] = btn
+
+  @classmethod
+  def createBtn(cls, btnOpts: PrjParam, baseBtn: QtWidgets.QAbstractButton=None, asToolBtn=False,
+                parent=None, **kwargs):
+    if asToolBtn:
+      btnType = QtWidgets.QToolButton
+    else:
+      btnType = QtWidgets.QPushButton
+    tooltipText = btnOpts.helpText
+    if baseBtn is not None:
+      newBtn = baseBtn
+    else:
+      newBtn = btnType(parent)
+      newBtn.setText(btnOpts.name)
+    if 'icon' in btnOpts.opts:
+      newBtn.setText('')
+      newBtn.setIcon(QtGui.QIcon(str(btnOpts.opts['icon'])))
+      tooltipText = btnOpts.addHelpText(btnOpts.name)
+    newBtn.setToolTip(tooltipText)
+    reg = params.pgregistered.ShortcutParameter.REGISTRY
+    if reg is not None:
+      reg.registerButton(btnOpts, newBtn, **kwargs)
+    return newBtn
+
+  def callFuncByParam(self, param: PrjParam):
+    if param is None:
+      return
+    # Ensure function is called in the event it requires a button to be checked
+    btn = self.paramToBtnMapping[param]
+    if btn.isCheckable():
+      btn.setChecked(True)
+    self.paramToFuncMapping[param](param)
+    self.lastTriggered = param
+
+  def addByParam(self, param: Parameter, copy=True, **registerOpts):
+    """
+    Adds a button to a group based on the parameter. Also works for group params
+    that have an acttion nested.
+    """
+    for param in params.parameditor.params_flattened(param):
+      curCopy = copy
+      if param.type() in ['action', 'shortcut'] and param.opts.get('guibtn', True):
+        existingBtn = None
+        try:
+          existingBtn = next(iter(param.items)).button
+        except (StopIteration, AttributeError):
+          curCopy = True
+        if curCopy:
+          self.create_addBtn(PrjParam(**param.opts), lambda *args: param.activate(),
+                             **registerOpts)
+        else:
+          self.addBtn(PrjParam(**param.opts), existingBtn, existingBtn.click)
+
+  @classmethod
+  def fromToolsEditors(cls,
+                       editors: t.Sequence[params.ParamEditor],
+                       title='Tools',
+                       ownerClctn: ButtonCollection=None,
+                       **registerOpts):
+    if ownerClctn is None:
+      ownerClctn = cls(title=title, exclusive=True)
+
+    for editor in editors:
+      ownerClctn.addByParam(editor.params, **registerOpts)
+
+    return ownerClctn
+
+  def toolbarFormat(self):
+    """
+    Returns a list of buttons + title in a format that's easier to add to a toolbar, e.g.
+    doesn't require as much horizontal space
+    """
+    title = self.title()
+    out: t.List[QtWidgets.QWidget] = [] if title is None else [QtWidgets.QLabel(self.title())]
+    for btn in self.paramToBtnMapping.values():
+      out.append(btn)
+    return out
+
+_layoutTypes = t.Union[Literal['H'], Literal['V']]
 class EasyWidget:
-  def __init__(self, children: Sequence[Union[QtWidgets.QWidget, EasyWidget]],
+  def __init__(self, children: t.Sequence[t.Union[QtWidgets.QWidget, EasyWidget]],
                layout: str=None, useSplitter=False, baseWidget: QtWidgets.QWidget=None):
     if baseWidget is None:
       baseWidget = QtWidgets.QWidget()
@@ -260,7 +409,7 @@ class EasyWidget:
     for ii, child in enumerate(self.children_):
       self.addChild(child, chSuggested)
 
-  def addChild(self, child: Union[QtWidgets.QWidget, Sequence, EasyWidget], suggestedLayout:str=None):
+  def addChild(self, child: t.Union[QtWidgets.QWidget, t.Sequence, EasyWidget], suggestedLayout:str=None):
     if isinstance(child, QtWidgets.QWidget):
       self.layout_.addWidget(child)
     else:
@@ -270,7 +419,7 @@ class EasyWidget:
       self.layout_.addWidget(child.widget_)
 
   @classmethod
-  def listChildrenWrapper(cls, children: Union[Sequence, EasyWidget], maybeNewLayout: str=None):
+  def listChildrenWrapper(cls, children: t.Union[t.Sequence, EasyWidget], maybeNewLayout: str=None):
     if not isinstance(children, EasyWidget):
       children = cls(children)
     if children.layout_ is None and maybeNewLayout is not None:
@@ -278,10 +427,10 @@ class EasyWidget:
     return children
 
   @classmethod
-  def buildMainWin(cls, children: Union[Sequence, EasyWidget], win: QtWidgets.QMainWindow=None, layout='V', **kwargs):
+  def buildMainWin(cls, children: t.Union[t.Sequence, EasyWidget], win: QtWidgets.QMainWindow=None, layout='V', **kwargs):
     if win is None:
       win = QtWidgets.QMainWindow()
-    if isinstance(children, Sequence):
+    if isinstance(children, t.Sequence):
       children = cls(children, layout=layout, **kwargs)
 
     children.build()
@@ -291,7 +440,7 @@ class EasyWidget:
     return win
 
   @classmethod
-  def buildWidget(cls, children: Union[Sequence, EasyWidget], layout='V', **kwargs):
+  def buildWidget(cls, children: t.Union[t.Sequence, EasyWidget], layout='V', **kwargs):
     builder = cls(children, layout=layout, **kwargs)
     builder.build()
     retWidget: HasEasyChild = builder.widget_
@@ -303,16 +452,18 @@ class HasEasyChild(QtWidgets.QMainWindow):
   easyChild: EasyWidget
 
 
-class ImgViewer(pg.PlotWidget):
+class ImgViewer(CompositionMixin, parameditor.EditorPropsMixin, pg.PlotWidget):
+
+
   @classmethod
   def __initEditorParams__(cls):
-    super().__initEditorParams__()
+    cls.toolsEditor = parameditor.ParamEditor.buildClsToolsEditor(cls, 'Region Tools')
 
   def __init__(self, imgSrc: np.ndarray=None, **kwargs):
     super().__init__(**kwargs)
-
     vb = self.getViewBox()
     self.menu: QtWidgets.QMenu = vb.menu
+    self.menu.clear()
     self.oldVbMenu: ViewBoxMenu = vb.menu
     # Disable default menus
     self.plotItem.ctrlMenu = None
@@ -328,8 +479,6 @@ class ImgViewer(pg.PlotWidget):
     self.imgItem = self.exposes(pg.ImageItem())
     self.imgItem.setZValue(-100)
     self.addItem(self.imgItem)
-    self.toolsEditor.registerFunc(lambda: self.oldVbMenu.viewAll.trigger(),
-                                  name='Reset Zoom')
     if imgSrc is not None:
       self.setImage(imgSrc)
 
@@ -355,6 +504,25 @@ class ImgViewer(pg.PlotWidget):
       fontColor = 'white'
     self.pxColor.setStyleSheet(f'background:rgba{tuple(pxColor)}; color:{fontColor}')
 
+  def setImage(self, imgSrc: t.Union[FilePath, np.ndarray]=None):
+    """
+    Allows the user to change the main image either from a filepath or array data
+    """
+    if isinstance(imgSrc, FilePath.__args__):
+      # TODO: Handle alpha channel images. For now, discard that data
+      imgSrc = cv.imread(str(imgSrc), cv.IMREAD_UNCHANGED)
+      if imgSrc.ndim > 3:
+        # Alpha channels cause unexpected results for most image processors. Avoid this
+        # by chopping it off until otherwise needed
+        imgSrc = imgSrc[:,:,0:3]
+      if imgSrc.ndim == 3:
+        imgSrc = cv.cvtColor(imgSrc, cv.COLOR_BGR2RGB)
+
+    if imgSrc is None:
+      self.imgItem.clear()
+    else:
+      self.imgItem.setImage(imgSrc)
+
 
 # class ImgCompositor(ImgViewer):
 #   def __init__(self, img: np.ndarray=None):
@@ -497,7 +665,7 @@ class PandasTableModel(QtCore.QAbstractTableModel):
         return value
     return None
 
-  def setData(self, index: QtCore.QModelIndex, value: Any, role:int=QtCore.Qt.ItemDataRole) -> bool:
+  def setData(self, index: QtCore.QModelIndex, value: t.Any, role:int=QtCore.Qt.ItemDataRole) -> bool:
     super().setData(index, role)
     row = index.row()
     col = index.column()
@@ -563,7 +731,7 @@ class PandasTableModel(QtCore.QAbstractTableModel):
       self.sigCompsChanged.emit(toEmit)
     return toEmit
 
-  def removeDfRows(self, idsToRemove: Sequence[int], emitChange=True):
+  def removeDfRows(self, idsToRemove: t.Sequence[int], emitChange=True):
     toEmit = self.defaultEmitDict.copy()
     # Generate ID list
     existingCompIds = self.compDf.index
